@@ -66,7 +66,7 @@ class URLMapTest(TestCase):
     def setUp(self):
         self.site = Site.objects.get(id=1)
         self.url = models.URLMap(site=self.site, path='/test_path')
-        self.hexdigest = '389661d2e64f9d426ad306abe6e8f957'
+        self.hexdigest = 'a6dd1406d4e5aadaafed9c2d285d36bd'
         self.cache_key = settings.URLOGRAPHER_CACHE_PREFIX + self.hexdigest
         self.mox = mox.Mox()
         self.mox.StubOutWithMock(models.URLMapManager, 'cached_get')
@@ -195,12 +195,11 @@ class URLMapTest(TestCase):
         self.mox.VerifyAll()
 
 
-
 class URLMapManagerTest(TestCase):
     def setUp(self):
-        self.site = Site(domain='example.com')
+        self.site = Site.objects.get(id=1)
         self.url = models.URLMap(site=self.site, path='/test_path')
-        self.hexdigest = '389661d2e64f9d426ad306abe6e8f957'
+        self.hexdigest = 'a6dd1406d4e5aadaafed9c2d285d36bd'
         self.cache_key = settings.URLOGRAPHER_CACHE_PREFIX + self.hexdigest
         self.mox = mox.Mox()
 
@@ -256,17 +255,20 @@ class URLMapManagerTest(TestCase):
         self.mox.VerifyAll()
         self.assertEqual(url, self.url)
 
-    @override_settings(URLOGRAPHER_INDEX_ALIASES=['index.html'])
+    @override_settings(URLOGRAPHER_INDEX_ALIASES=['index.html'],
+                       URLOGRAPHER_CACHE_PREFIX='urlographer')
     def test_cached_get_index_alias_cache_hit(self):
         index_urlmap = models.URLMap(site=self.site, path='/index.html',
-                                     status_code=204)
-        index_urlmap.set_hexdigest()
-        index_key = settings.URLOGRAPHER_CACHE_PREFIX + index_urlmap.hexdigest
-        root_key = (settings.URLOGRAPHER_CACHE_PREFIX +
-                    '617a9471507f0eb608f3858291adb70f')
+                                     status_code=204, hexdigest='index1234')
+        self.mox.StubOutWithMock(models.URLMap, 'set_hexdigest')
+        self.mox.StubOutWithMock(models.URLMap, 'cache_key')
         self.mox.StubOutWithMock(models.cache, 'get')
-        models.cache.get(root_key)
-        models.cache.get(index_key).AndReturn(index_urlmap)
+        models.URLMap.set_hexdigest()
+        models.URLMap.cache_key().AndReturn('urlographer:root1234')
+        models.cache.get('urlographer:root1234')
+        models.URLMap.set_hexdigest()
+        models.URLMap.cache_key().AndReturn('urlographer:index1234')
+        models.cache.get('urlographer:index1234').AndReturn(index_urlmap)
         self.mox.ReplayAll()
         urlmap = models.URLMap.objects.cached_get(self.site, '/')
         self.mox.VerifyAll()
